#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var commander_1 = require("commander");
var global_1 = require("../lib/core/global");
var bst_virtual_alexa_1 = require("../lib/client/bst-virtual-alexa");
var logging_helper_1 = require("../lib/core/logging-helper");
var chalk = require("chalk");
commander_1.program.version(global_1.Global.version());
commander_1.program
    .usage("[options] <Intent> [SlotName=SlotValue...]")
    .option("-u, --url <alexa-skill-url>", "The URL of the Alexa skill to send to - defaults to current proxied skill")
    .option("-l, --locale <locale>", "The locale expected for the Alexa skill - defaults to en-US")
    .option("-m, --model <interaction-model-path>", "Path to the interaction model file - defaults to ./models/en-US.json")
    .option("-i, --intents <intent-schema-path>", "Path to the intent schema file - defaults to ./speechAssets/IntentSchema.json")
    .option("-s, --samples <sample-utterances-path>", "Path to the sample utterances file - defaults to ./speechAssets/SampleUtterances.txt")
    .option("-a, --appId <application-id>", "The application ID for the skill")
    .option("-U, --userId <user-id>", "Sets the user id to the specified value")
    .option("-t, --accessToken <accessToken>", "Sets the access token for emulating a user with a linked account")
    .option("-n, --newSession", "Starts a new session")
    .description("Creates an intent request based on the specified intent and sends it to your skill")
    .action(function () {
    var intentName = commander_1.program.args[0];
    var slots = {};
    for (var i = 1; i < commander_1.program.args.length; i++) {
        var slotArg = commander_1.program.args[i];
        if (typeof slotArg !== "string") {
            continue;
        }
        if (slotArg.indexOf("=") === -1) {
            console.error("Invalid slot specified: " + slotArg + ". Must be in the form SlotName=SlotValue");
            console.error();
            process.exit(0);
            return;
        }
        var slotName = slotArg.split("=")[0];
        var slotValue = slotArg.split("=")[1];
        slots[slotName] = slotValue;
    }
    var options = commander_1.program.opts();
    var url = options.url;
    var interactionModel = options.model;
    var intentSchemaPath = options.intents;
    var samplesPath = options.samples;
    var applicationID = options.appId;
    var locale = options.locale;
    var newSession = options.newSession;
    var userId = options.userId;
    if (options.url === undefined) {
        var proxyProcess = global_1.Global.running();
        if (proxyProcess === null) {
            console.log("No URL specified and no proxy is currently running");
            console.log("");
            console.log("URL (--url) must be specified if no proxy is currently running");
            console.log("");
            console.log("If a proxy is running, intents will automatically be sent to it");
            process.exit(0);
            return;
        }
        url = "http://localhost:" + proxyProcess.port;
    }
    var savedSession = global_1.Global.config().loadSession() || {};
    var localeToUse = locale ? locale : savedSession.locale;
    var userIdToUse = userId ? userId : savedSession.userId;
    var speaker = new bst_virtual_alexa_1.BSTVirtualAlexa(url, interactionModel, intentSchemaPath, samplesPath, applicationID, localeToUse, userIdToUse);
    try {
        speaker.start();
    }
    catch (error) {
        process.exit(0);
        return;
    }
    if (userIdToUse) {
        speaker.context().user().setID(userIdToUse);
    }
    if (options.accessToken) {
        speaker.context().setAccessToken(options.accessToken);
    }
    try {
        if (newSession) {
            speaker.deleteSession();
        }
        speaker.intended(intentName, slots, function (error, response, request) {
            console.log("Intended: " + intentName);
            console.log("");
            if (request) {
                console.log("Request:");
                console.log(chalk.hex(logging_helper_1.LoggingHelper.REQUEST_COLOR)(JSON.stringify(request, null, 4)));
                console.log("");
            }
            if (error) {
                console.log(chalk.red("Error: " + error.message));
                return;
            }
            var jsonPretty = JSON.stringify(response, null, 4);
            console.log("Response:");
            console.log(chalk.cyan(jsonPretty));
            console.log("");
        });
    }
    catch (e) {
        console.error(chalk.red("Error with intent:"));
        console.error(chalk.red(e.message));
        console.error();
    }
});
if (process.argv.slice(2).length === 0) {
    commander_1.program.outputHelp();
}
global_1.Global.initializeCLI().then(function () { return commander_1.program.parse(process.argv); });
//# sourceMappingURL=bst-intend.js.map